==> ubuntu-core-upgrader_0.7.3.tar.xz: ubuntu-core-upgrader-0.7.3/ubuntucoreupgrader/apply.py <==
import json
import os
import select
import subprocess
import time

from systemimage.config import config


class ApplyUpgradeError(Exception):
    pass


def is_sym_link_broken(path):
    '''
    Returns True if the specified path is a broken symbolic link, else
    False.
    '''
    try:
        os.lstat(path)
        os.stat(path)
    except:
        return True
    return False


def newest_mtime(directory):
    """
    Find the most recently modified file in a directory.
    Ignores broken symlinks.
    """
    files = [f for f in os.listdir(directory)
             if not is_sym_link_broken(os.path.join(directory, f))]

    return max([os.path.getmtime(os.path.join(directory, f)) for f in files])


class ApplyUpgrade:
    """Hook for system-image-cli"""

    UPGRADER = ["ubuntu-core-upgrade"]
    UPGRADER_ARGS = ["/writable/cache/ubuntu_command"]

    def apply_update(self):
        p = subprocess.Popen(
            self.UPGRADER + self.UPGRADER_ARGS,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        # collect the output while iterating to avoid hitting the 64k
        # pipe size limit (unlikely but...)
        stdout = ""
        while p.poll() is None:
            # FIXME: dbus is history soon
            if config.dbus_service:
                config.dbus_service.UpdateProgress(-1, -1)
            # send alive ping
            if getattr(config, "machine_readable", False):
                print(json.dumps({"type": "spinner",
                                  "msg": "Applying update"}))
            time.sleep(0.1)
            if select.select([p.stdout.fileno()], [], []) != ([], [], []):
                stdout += p.stdout.read().decode("utf-8", errors="replace")
        # get return code
        ret_code = p.poll()
        # close the pipes
        p.communicate()
        if ret_code != 0:
            raise ApplyUpgradeError(
                "%s exited with %s:\n%s", self.UPGRADER, ret_code, stdout)
        return True

    # FIXME: this is the entry point that system-image-cli uses, we need
    #        to teach it that the name should be something more generic
    #        than reboot
    #
    # Note that this reboot logic is specific to in-place upgrades.
    def reboot(self):
        old_mtime_in_boot = newest_mtime("/boot")
        self.apply_update()
        if newest_mtime("/boot") > old_mtime_in_boot:
            print("Need reboot")
            subprocess.check_call(["/sbin/reboot"])

==> ubuntu-core-upgrader_0.7.10.tar.xz: ubuntu-core-upgrader-0.7.10/ubuntucoreupgrader/apply.py <==
import json
import os
import select
import subprocess
import sys

from systemimage.config import config


class ApplyUpgradeError(Exception):
    pass


def is_sym_link_broken(path):
    '''
    Returns True if the specified path is a broken symbolic link, else
    False.
    '''
    try:
        os.lstat(path)
        os.stat(path)
    except:
        return True
    return False


class ApplyUpgrade:
    """Hook for system-image-cli"""

    UPGRADER = ["ubuntu-core-upgrade"]
    UPGRADER_ARGS = ["/writable/cache/ubuntu_command"]

    def apply_update(self):
        p = subprocess.Popen(
            self.UPGRADER + self.UPGRADER_ARGS,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        # collect the output while iterating to avoid hitting the 64k
        # pipe size limit (unlikely but...)
        stdout = ""
        while p.poll() is None:
            # send alive ping
            if getattr(config, "machine_readable", False):
                # ensure its written out
                sys.stdout.write(json.dumps({"type": "spinner",
                                             "msg": "Applying update"})+"\n")
                sys.stdout.flush()
            if select.select([p.stdout.fileno()], [], [], 0.1) != ([], [], []):
                stdout += p.stdout.read(1).decode("utf-8", errors="replace")
        # get return code
        ret_code = p.poll()
        # close the pipes
        p.communicate()
        if ret_code != 0:
            raise ApplyUpgradeError(
                "%s exited with %s:\n%s", self.UPGRADER, ret_code, stdout)
        return True

    # entry point for s-i 2.x
    def reboot(self):
        self.apply_update()

    # entry point for s-i 3.0
    def apply(self):
        self.apply_update()
